@@ -31,16 +31,19 @@
 #	include <config.h>
 #endif
 
-#include <synfig/general.h>
 
-#include "renddesc.h"
 #include <gtkmm/label.h>
 #include <gtkmm/frame.h>
 #include <gtkmm/alignment.h>
 #include <gtkmm/box.h>
 #include <gtkmm/grid.h>
+#include <gtkmm/drawingarea.h>
+
 #include <ETL/misc>
-//#include <gtkmm/separator.h>
+
+#include <synfig/general.h>
+
+#include "renddesc.h"
 
 #include <gui/localization.h>
 
@@ -77,33 +80,158 @@ using namespace studio;
 
 /* === P R O C E D U R E S ================================================= */
 
+
+class GammaPattern : public Gtk::DrawingArea
+{
+private:
+	Gamma gamma;
+	int tile_w, tile_h, gradient_h;
+	Cairo::RefPtr<Cairo::SurfacePattern> pattern;
+
+public:
+	GammaPattern():
+		tile_w(80),
+		tile_h(80),
+		gradient_h(20)
+	{
+		set_size_request(tile_w*4, tile_h*2 + gradient_h*2);
+
+		// make pattern
+		Cairo::RefPtr<Cairo::ImageSurface> surface =
+			Cairo::ImageSurface::create(Cairo::FORMAT_ARGB32, 2, 2);
+		Cairo::RefPtr<Cairo::Context> context = Cairo::Context::create(surface);
+		context->set_operator(Cairo::OPERATOR_SOURCE);
+		context->set_source_rgba(0, 0, 0, 0);
+		context->rectangle(0, 0, 1, 1);
+		context->fill();
+		context->set_source_rgba(0, 0, 0, 1);
+		context->rectangle(0, 0, 1, 1);
+		context->rectangle(1, 1, 1, 1);
+		context->fill();
+		surface->flush();
+
+		pattern = Cairo::SurfacePattern::create(surface);
+		pattern->set_filter(Cairo::FILTER_NEAREST);
+		pattern->set_extend(Cairo::EXTEND_REPEAT);
+	}
+
+	const Gamma& get_gamma() const { return gamma; }
+
+	void set_gamma(const Gamma &x) {
+		if (gamma == x) return;
+		gamma = x;
+		queue_draw();
+	}
+	
+	virtual bool on_draw(const Cairo::RefPtr<Cairo::Context> &cr)
+	{
+		cr->save();
+		
+		// prepare colors
+		ColorReal values[3] = {0.25, 0.5, 1};
+		Color colors[3][4];
+		for(int i = 0; i < 3; ++i) {
+			ColorReal v = values[i];
+			Color *c = colors[i];
+			c[0] = Color(v, v, v);
+			c[1] = Color(v, 0, 0);
+			c[2] = Color(0, v, 0);
+			c[3] = Color(0, 0, v);
+			for(int j = 0; j < 4; ++j)
+				c[j] = gamma.apply(c[j]);
+		}
+		Color *gray25 = colors[0];
+		Color *gray50 = colors[1];
+		Color *white  = colors[2];
+		
+		// 50% pattern
+		for(int i = 0; i < 4; ++i)
+		{
+			cr->set_source_rgb(white[i].get_r(), white[i].get_g(), white[i].get_b());
+			cr->rectangle(i*tile_w, 0, tile_w, tile_h);
+			cr->fill();
+
+			cr->set_source(pattern);
+			cr->rectangle(i*tile_w, 0, tile_w, tile_h);
+			cr->fill();
+
+			cr->set_source_rgb(gray50[i].get_r(), gray50[i].get_g(), gray50[i].get_b());
+			cr->rectangle(i*tile_w+tile_w/4, tile_h/4, tile_w-tile_w/2, tile_h-tile_h/2);
+			cr->fill();
+		}
+
+		// 25% pattern
+		for(int i = 0; i < 4; ++i)
+		{
+			cr->set_source_rgb(gray50[i].get_r(), gray50[i].get_g(), gray50[i].get_b());
+			cr->rectangle(i*tile_w, tile_h, tile_w, tile_h);
+			cr->fill();
+
+			cr->set_source(pattern);
+			cr->rectangle(i*tile_w, tile_h, tile_w, tile_h);
+			cr->fill();
+
+			cr->set_source_rgb(gray25[i].get_r(), gray25[i].get_g(), gray25[i].get_b());
+			cr->rectangle(i*tile_w+tile_w/4, tile_h+tile_h/4, tile_w-tile_w/2, tile_h-tile_h/2);
+			cr->fill();
+		}
+
+		// black and white level pattern
+		cr->set_source_rgb(0, 0, 0);
+		cr->rectangle(0, tile_h*2, tile_w*4, gradient_h);
+		cr->fill();
+		cr->set_source_rgb(1, 1, 1);
+		cr->rectangle(0, tile_h*2 + gradient_h, tile_w*4, gradient_h);
+		cr->fill();
+		ColorReal level = 1;
+		for(int i = 0; i < 8; ++i)
+		{
+			level *= 0.5;
+			Color black = gamma.apply(Color(level, level, level));
+			Color white = gamma.apply(Color(1-level, 1-level, 1-level));
+			double x = tile_w*4*(i/8.0 + 1/16.0);
+			double yb = tile_h*2 + gradient_h/2.0;
+			double yw = yb + gradient_h;
+			double r = gradient_h/4.0;
+			
+			cr->set_source_rgb(black.get_r(), black.get_g(), black.get_b());
+			cr->arc(x, yb, r, 0, 2*M_PI);
+			cr->fill();
+
+			cr->set_source_rgb(white.get_r(), white.get_g(), white.get_b());
+			cr->arc(x, yw, r, 0, 2*M_PI);
+			cr->fill();
+		}
+		cr->restore();
+		return true;
+	}
+}; // END of class GammaPattern
+
+
 /* === M E T H O D S ======================================================= */
 
+
 Widget_RendDesc::Widget_RendDesc():
 	Gtk::Notebook(),
-	adjustment_width(Gtk::Adjustment::create(1,1,SYNFIG_MAX_PIXEL_WIDTH)),
-	adjustment_height(Gtk::Adjustment::create(1,1,SYNFIG_MAX_PIXEL_HEIGHT)),
-	adjustment_xres(Gtk::Adjustment::create(0,0.0000000001,10000000)),
-	adjustment_yres(Gtk::Adjustment::create(0,0.0000000001,10000000)),
-	adjustment_phy_width(Gtk::Adjustment::create(0,0.0000000001,10000000)),
-	adjustment_phy_height(Gtk::Adjustment::create(0,0.0000000001,10000000)),
-	adjustment_fps(Gtk::Adjustment::create(0,0.0000000001,10000000)),
-	adjustment_span(Gtk::Adjustment::create(0,0.0000000001,10000000))
+	adjustment_width     (Gtk::Adjustment::create(1, 1, SYNFIG_MAX_PIXEL_WIDTH)),
+	adjustment_height    (Gtk::Adjustment::create(1, 1, SYNFIG_MAX_PIXEL_HEIGHT)),
+	adjustment_xres      (Gtk::Adjustment::create(0, 1e-10, 1e7)),
+	adjustment_yres      (Gtk::Adjustment::create(0, 1e-10, 1e7)),
+	adjustment_phy_width (Gtk::Adjustment::create(0, 1e-10, 1e7)),
+	adjustment_phy_height(Gtk::Adjustment::create(0, 1e-10, 1e7)),
+	adjustment_fps       (Gtk::Adjustment::create(0, 1e-10, 1e7)),
+	adjustment_span      (Gtk::Adjustment::create(0, 1e-10, 1e7)),
+	adjustment_gamma_r   (Gtk::Adjustment::create(1, 0.1, 3.0, 0.025, 0.025)),
+	adjustment_gamma_g   (Gtk::Adjustment::create(1, 0.1, 3.0, 0.025, 0.025)),
+	adjustment_gamma_b   (Gtk::Adjustment::create(1, 0.1, 3.0, 0.025, 0.025))
 {
 	update_lock=0;
-
 	create_widgets();
 	connect_signals();
-
-	Gtk::Label *image_tab_label = manage(new Gtk::Label(_("Image")));
-	Gtk::Label *time_tab_label = manage(new Gtk::Label(_("Time")));
-	Gtk::Label *other_tab_label = manage(new Gtk::Label(_("Other")));
-	Gtk::Widget *imageTab = create_image_tab();
-	Gtk::Widget *timeTab = create_time_tab();
-	Gtk::Widget *otherTab = create_other_tab();
-	append_page(*imageTab, *image_tab_label);
-	append_page(*timeTab, *time_tab_label);
-	append_page(*otherTab, *other_tab_label);
+	append_page(*create_image_tab(), Glib::ustring(_("Image")));
+	append_page(*create_time_tab(),  Glib::ustring(_("Time")));
+	append_page(*create_gamma_tab(), Glib::ustring(_("Gamma correction")));
+	append_page(*create_other_tab(), Glib::ustring(_("Other")));
 }
 
 Widget_RendDesc::~Widget_RendDesc()
@@ -113,7 +241,6 @@ Widget_RendDesc::~Widget_RendDesc()
 void Widget_RendDesc::set_rend_desc(const synfig::RendDesc &rend_desc)
 {
 	if(update_lock)return;
-
 	rend_desc_=rend_desc;
 	refresh();
 }
@@ -122,7 +249,8 @@ void
 Widget_RendDesc::refresh()
 {
 	UpdateLock lock(update_lock);
-	//Image Tab
+
+	// image tab
 	adjustment_width->set_value(rend_desc_.get_w());
 	adjustment_height->set_value(rend_desc_.get_h());
 	adjustment_phy_width->set_value(METERS2INCHES(rend_desc_.get_physical_w()));
@@ -135,24 +263,29 @@ Widget_RendDesc::refresh()
 	int w_ratio, h_ratio;
 	rend_desc_.get_pixel_ratio_reduced(w_ratio, h_ratio);
 	std::ostringstream px_ratio_str;
-	px_ratio_str << _("Image Size Ratio : ") << w_ratio << '/' << h_ratio;
+	px_ratio_str << _("Image Size Ratio: ") << w_ratio << '/' << h_ratio;
 	pixel_ratio_label->set_label(px_ratio_str.str());
 
 	entry_tl->set_value(rend_desc_.get_tl());
 	entry_br->set_value(rend_desc_.get_br());
 	adjustment_span->set_value(rend_desc_.get_span());
 
-	//Time Tab
+	// time tab
 	entry_start_time->set_fps(rend_desc_.get_frame_rate());
 	entry_start_time->set_value(rend_desc_.get_time_start());
 	entry_end_time->set_fps(rend_desc_.get_frame_rate());
 	entry_end_time->set_value(rend_desc_.get_time_end());
 	entry_duration->set_fps(rend_desc_.get_frame_rate());
 	entry_duration->set_value(rend_desc_.get_duration());
-
 	adjustment_fps->set_value(rend_desc_.get_frame_rate());
 
-	//Other Tab
+	// gamma tab
+	adjustment_gamma_r->set_value(rend_desc_.get_gamma().get_r());
+	adjustment_gamma_g->set_value(rend_desc_.get_gamma().get_g());
+	adjustment_gamma_b->set_value(rend_desc_.get_gamma().get_b());
+	dynamic_cast<GammaPattern*>(gamma_pattern)->set_gamma(rend_desc_.get_gamma().get_inverted());
+	
+	// other tab
 	toggle_px_aspect->set_active((bool)(rend_desc_.get_flags()&RendDesc::PX_ASPECT));
 	toggle_px_width->set_active((bool)(rend_desc_.get_flags()&RendDesc::PX_W));
 	toggle_px_height->set_active((bool)(rend_desc_.get_flags()&RendDesc::PX_H));
@@ -281,7 +414,6 @@ Widget_RendDesc::on_end_time_changed()
 	signal_changed()();
 }
 
-
 void
 Widget_RendDesc::on_duration_changed()
 {
@@ -342,6 +474,19 @@ Widget_RendDesc::on_span_changed()
 	signal_changed()();
 }
 
+void
+Widget_RendDesc::on_gamma_changed()
+{
+	if(update_lock)return;
+	UpdateLock lock(update_lock);
+	rend_desc_.set_gamma(Gamma(
+		adjustment_gamma_r->get_value(),
+		adjustment_gamma_g->get_value(),
+		adjustment_gamma_b->get_value() ));
+	refresh();
+	signal_changed()();
+}
+
 void
 Widget_RendDesc::disable_time_section()
 {
@@ -354,6 +499,18 @@ Widget_RendDesc::enable_time_section()
 	time_frame->set_sensitive(true);
 }
 
+void
+Widget_RendDesc::disable_gamma_section()
+{
+	gamma_frame->set_sensitive(false);
+}
+
+void
+Widget_RendDesc::enable_gamma_section()
+{
+	gamma_frame->set_sensitive(true);
+}
+
 void
 Widget_RendDesc::on_ratio_wh_toggled()
 {
@@ -364,10 +521,12 @@ Widget_RendDesc::on_ratio_wh_toggled()
 	{
 		rend_desc_.set_pixel_ratio(adjustment_width->get_value(), adjustment_height->get_value());
 		rend_desc_.set_flags(rend_desc_.get_flags()|RendDesc::LINK_IM_ASPECT);
+		rend_desc_.set_flags(rend_desc_.get_flags()&~RendDesc::PX_ASPECT);
 	}
 	else
 	{
 		rend_desc_.set_flags(rend_desc_.get_flags()&~RendDesc::LINK_IM_ASPECT);
+		rend_desc_.set_flags(rend_desc_.get_flags()|RendDesc::PX_ASPECT);
 	}
 }
 
@@ -414,6 +573,12 @@ Widget_RendDesc::create_widgets()
 	entry_end_time=manage(new Widget_Time());
 	entry_duration=manage(new Widget_Time());
 	entry_focus=manage(new Widget_Vector());
+	entry_gamma_r=manage(new Gtk::SpinButton(adjustment_gamma_r,0.01,2));
+	entry_gamma_g=manage(new Gtk::SpinButton(adjustment_gamma_g,0.01,2));
+	entry_gamma_b=manage(new Gtk::SpinButton(adjustment_gamma_b,0.01,2));
+	scale_gamma_r=manage(new Gtk::Scale(adjustment_gamma_r));
+	scale_gamma_g=manage(new Gtk::Scale(adjustment_gamma_g));
+	scale_gamma_b=manage(new Gtk::Scale(adjustment_gamma_b));
 	toggle_px_aspect=manage(new Gtk::CheckButton(_("_Pixel Aspect"), true));
 	toggle_px_aspect->set_alignment(0, 0.5);
 	toggle_px_width=manage(new Gtk::CheckButton(_("Pi_xel Width"), true));
@@ -438,29 +603,32 @@ Widget_RendDesc::create_widgets()
 void
 Widget_RendDesc::connect_signals()
 {
-	entry_width->signal_value_changed().connect(sigc::mem_fun(*this,&studio::Widget_RendDesc::on_width_changed));
-	entry_height->signal_value_changed().connect(sigc::mem_fun(*this,&studio::Widget_RendDesc::on_height_changed));
-	entry_xres->signal_value_changed().connect(sigc::mem_fun(*this,&studio::Widget_RendDesc::on_xres_changed));
-	entry_yres->signal_value_changed().connect(sigc::mem_fun(*this,&studio::Widget_RendDesc::on_yres_changed));
-	entry_phy_width->signal_value_changed().connect(sigc::mem_fun(*this,&studio::Widget_RendDesc::on_phy_width_changed));
-	entry_phy_height->signal_value_changed().connect(sigc::mem_fun(*this,&studio::Widget_RendDesc::on_phy_height_changed));
-	entry_span->signal_value_changed().connect(sigc::mem_fun(*this,&studio::Widget_RendDesc::on_span_changed));
-	entry_tl->signal_value_changed().connect(sigc::mem_fun(*this,&studio::Widget_RendDesc::on_tl_changed));
-	entry_br->signal_value_changed().connect(sigc::mem_fun(*this,&studio::Widget_RendDesc::on_br_changed));
-	entry_fps->signal_value_changed().connect(sigc::mem_fun(*this,&studio::Widget_RendDesc::on_fps_changed));
-	entry_start_time->signal_value_changed().connect(sigc::mem_fun(*this,&studio::Widget_RendDesc::on_start_time_changed));
-	entry_end_time->signal_value_changed().connect(sigc::mem_fun(*this,&studio::Widget_RendDesc::on_end_time_changed));
-	entry_duration->signal_value_changed().connect(sigc::mem_fun(*this,&studio::Widget_RendDesc::on_duration_changed));
-	entry_focus->signal_value_changed().connect(sigc::mem_fun(*this,&studio::Widget_RendDesc::on_focus_changed));
+	entry_width     ->signal_value_changed().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_width_changed));
+	entry_height    ->signal_value_changed().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_height_changed));
+	entry_xres      ->signal_value_changed().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_xres_changed));
+	entry_yres      ->signal_value_changed().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_yres_changed));
+	entry_phy_width ->signal_value_changed().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_phy_width_changed));
+	entry_phy_height->signal_value_changed().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_phy_height_changed));
+	entry_span      ->signal_value_changed().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_span_changed));
+	entry_tl        ->signal_value_changed().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_tl_changed));
+	entry_br        ->signal_value_changed().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_br_changed));
+	entry_fps       ->signal_value_changed().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_fps_changed));
+	entry_start_time->signal_value_changed().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_start_time_changed));
+	entry_end_time  ->signal_value_changed().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_end_time_changed));
+	entry_duration  ->signal_value_changed().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_duration_changed));
+	entry_focus     ->signal_value_changed().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_focus_changed));
+	entry_gamma_r   ->signal_value_changed().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_gamma_changed));
+	entry_gamma_g   ->signal_value_changed().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_gamma_changed));
+	entry_gamma_b   ->signal_value_changed().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_gamma_changed));
+
 	toggle_px_aspect->signal_toggled().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_lock_changed));
-	toggle_px_width->signal_toggled().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_lock_changed));
+	toggle_px_width ->signal_toggled().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_lock_changed));
 	toggle_px_height->signal_toggled().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_lock_changed));
 	toggle_im_aspect->signal_toggled().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_lock_changed));
-	toggle_im_width->signal_toggled().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_lock_changed));
+	toggle_im_width ->signal_toggled().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_lock_changed));
 	toggle_im_height->signal_toggled().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_lock_changed));
-	toggle_im_span->signal_toggled().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_lock_changed));
-
-	toggle_wh_ratio->signal_toggled().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_ratio_wh_toggled));
+	toggle_im_span  ->signal_toggled().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_lock_changed));
+	toggle_wh_ratio ->signal_toggled().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_ratio_wh_toggled));
 	toggle_res_ratio->signal_toggled().connect(sigc::mem_fun(*this, &studio::Widget_RendDesc::on_ratio_res_toggled));
 }
 
@@ -580,46 +748,142 @@ Widget_RendDesc::create_time_tab()
 {
 	Gtk::Alignment *paddedPanel = manage(new Gtk::Alignment(0, 0, 1, 1));
 	paddedPanel->set_padding(12, 12, 12, 12);
-
+	
 	Gtk::Box *panelBox = manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 12));  // for future widgets
 	panelBox->set_homogeneous(false);
 	paddedPanel->add(*panelBox);
+	
+	{
+		Gtk::Frame *frame = manage(new Gtk::Frame(_("Time Settings")));
+		frame->set_shadow_type(Gtk::SHADOW_NONE);
+		((Gtk::Label*)frame->get_label_widget())->set_markup(_("<b>Time Settings</b>"));
+		panelBox->pack_start(*frame, Gtk::PACK_SHRINK);
+		time_frame = frame;
+		
+		Gtk::Alignment *framePadding = manage(new Gtk::Alignment(0, 0, 1, 1));
+		framePadding->set_padding(6, 0, 24, 0);
+		frame->add(*framePadding);
+		
+		Gtk::Grid *frameGrid = manage(new Gtk::Grid());
+		framePadding->add(*frameGrid);
+		frameGrid->set_row_spacing(6);
+		frameGrid->set_column_spacing(250);
+		int row = 0;
+		
+		{
+			Gtk::Label *label = manage(new Gtk::Label(_("_Frames per second"), 0, 0.5, true));
+			label->set_mnemonic_widget(*entry_fps);
+			frameGrid->attach(*label, 0, row, 1, 1);
+			entry_fps->set_hexpand(true);
+			frameGrid->attach(*entry_fps, 1, row++, 1, 1);
+		}
+		
+		{
+			Gtk::Label *label = manage(new Gtk::Label(_("_Start Time"), 0, 0.5, true));
+			label->set_mnemonic_widget(*entry_start_time);
+			frameGrid->attach(*label, 0, row, 1, 1);
+			frameGrid->attach(*entry_start_time, 1, row++, 1, 1);
+		}
+		
+		{
+			Gtk::Label *label = manage(new Gtk::Label(_("_End Time"), 0, 0.5, true));
+			label->set_mnemonic_widget(*entry_end_time);
+			frameGrid->attach(*label, 0, row, 1, 1);
+			frameGrid->attach(*entry_end_time, 1, row++, 1, 1);
+		}
+		
+		{
+			Gtk::Label *label = manage(new Gtk::Label(_("_Duration"), 0, 0.5, true));
+			label->set_mnemonic_widget(*entry_duration);
+			frameGrid->attach(*label, 0, row, 1, 1);
+			frameGrid->attach(*entry_duration, 1, row++, 1, 1);
+		}
+	}
+	
+	paddedPanel->show_all();
+	return paddedPanel;
+}
 
-	time_frame = manage(new Gtk::Frame(_("Time Settings")));
-	time_frame->set_shadow_type(Gtk::SHADOW_NONE);
-	((Gtk::Label *) time_frame->get_label_widget())->set_markup(_("<b>Time Settings</b>"));
-	panelBox->pack_start(*time_frame, Gtk::PACK_SHRINK);
+Gtk::Widget *
+Widget_RendDesc::create_gamma_tab()
+{
+	Gtk::Alignment *paddedPanel = manage(new Gtk::Alignment(0, 0, 1, 1));
+	paddedPanel->set_padding(12, 12, 12, 12);
 
-	Gtk::Alignment *timeFramePadding = manage(new Gtk::Alignment(0, 0, 1, 1));
-	timeFramePadding->set_padding(6, 0, 24, 0);
-	time_frame->add(*timeFramePadding);
-
-	Gtk::Grid *timeFrameGrid = manage(new Gtk::Grid());
-	timeFramePadding->add(*timeFrameGrid);
-	timeFrameGrid->set_row_spacing(6);
-	timeFrameGrid->set_column_spacing(250);
-
-	Gtk::Label *timeFPSLabel = manage(new Gtk::Label(_("_Frames per second"), 0, 0.5, true));
-	timeFPSLabel->set_mnemonic_widget(*entry_fps);
-	timeFrameGrid->attach(*timeFPSLabel, 0, 0, 1, 1);
-	entry_fps->set_hexpand(true);
-	timeFrameGrid->attach(*entry_fps, 1, 0, 1, 1);
-
-	Gtk::Label *timeStartLabel = manage(new Gtk::Label(_("_Start Time"), 0, 0.5, true));
-	timeStartLabel->set_mnemonic_widget(*entry_start_time);
-	timeFrameGrid->attach(*timeStartLabel, 0, 1, 1, 1);
-	timeFrameGrid->attach(*entry_start_time, 1, 1, 1, 1);
-
-	Gtk::Label *timeEndLabel = manage(new Gtk::Label(_("_End Time"), 0, 0.5, true));
-	timeEndLabel->set_mnemonic_widget(*entry_end_time);
-	timeFrameGrid->attach(*timeEndLabel, 0, 2, 1, 1);
-	timeFrameGrid->attach(*entry_end_time, 1, 2, 1, 1);
-
-	Gtk::Label *timeDurationLabel = manage(new Gtk::Label(_("_Duration"), 0, 0.5, true));
-	timeDurationLabel->set_mnemonic_widget(*entry_duration);
-	timeFrameGrid->attach(*timeDurationLabel, 0, 3, 1, 1);
-	timeFrameGrid->attach(*entry_duration, 1, 3, 1, 1);
+	Gtk::Box *panelBox = manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 12));
+	panelBox->set_homogeneous(false);
+	paddedPanel->add(*panelBox);
+	
+	{
+		Gtk::Frame *frame = manage(new Gtk::Frame(_("Gamma Correction Settings")));
+		frame->set_shadow_type(Gtk::SHADOW_NONE);
+		((Gtk::Label*)frame->get_label_widget())->set_markup(_("<b>Gamma Correction Settings</b>"));
+		panelBox->pack_start(*frame, Gtk::PACK_SHRINK);
 
+		Gtk::Alignment *framePadding = manage(new Gtk::Alignment(0, 0, 1, 1));
+		framePadding->set_padding(6, 0, 6, 0);
+		frame->add(*framePadding);
+		gamma_frame = frame;
+		
+		Gtk::Grid *frameGrid = manage(new Gtk::Grid());
+		framePadding->add(*frameGrid);
+		frameGrid->set_row_spacing(10);
+		frameGrid->set_column_spacing(10);
+		int row = 0;
+		int col = 0;
+		
+		gamma_pattern = manage(new GammaPattern());
+		gamma_pattern->set_halign(Gtk::ALIGN_START);
+		frameGrid->attach(*gamma_pattern, col, row, 1, 1);
+		
+		// place this grid within frameGrid on column 2
+		// for labels, scales and spin buttons
+		Gtk::Grid *controlsGrid = manage(new Gtk::Grid());
+		controlsGrid->set_row_spacing(10);
+		controlsGrid->set_column_spacing(5);
+		controlsGrid->set_column_homogeneous(false);
+		frameGrid->attach(*controlsGrid, col+1, row, 1, 1);
+		
+		{
+			Gtk::Label *label = manage(new Gtk::Label(_("_Red"), 0, 0.5, true));
+			label->set_mnemonic_widget(*entry_gamma_r);
+			controlsGrid->attach(*label, col, row, 1, 1);
+			scale_gamma_r->set_hexpand(true);
+			scale_gamma_r->set_draw_value(false);
+			scale_gamma_r->set_round_digits(1);
+			controlsGrid->attach(*scale_gamma_r, col+1, row, 1, 1);
+			//entry_gamma_r->set_hexpand(true);
+			entry_gamma_r->set_halign(Gtk::ALIGN_END);
+			controlsGrid->attach(*entry_gamma_r, col+2, row++, 1, 1);
+		}
+		
+		{
+			Gtk::Label *label = manage(new Gtk::Label(_("_Green"), 0, 0.5, true));
+			label->set_mnemonic_widget(*entry_gamma_g);
+			controlsGrid->attach(*label, col, row, 1, 1);
+			scale_gamma_g->set_hexpand(true);
+			scale_gamma_g->set_draw_value(false);
+			scale_gamma_g->set_round_digits(1);
+			controlsGrid->attach(*scale_gamma_g, col+1, row, 1, 1);
+			//entry_gamma_g->set_hexpand(true);
+			entry_gamma_g->set_halign(Gtk::ALIGN_END);
+			controlsGrid->attach(*entry_gamma_g, col+2, row++, 1, 1);
+		}
+		
+		{
+			Gtk::Label *label = manage(new Gtk::Label(_("_Blue"), 0, 0.5, true));
+			label->set_mnemonic_widget(*entry_gamma_b);
+			controlsGrid->attach(*label, col, row, 1, 1);
+			scale_gamma_b->set_hexpand(true);
+			scale_gamma_b->set_draw_value(false);
+			scale_gamma_b->set_round_digits(1);
+			controlsGrid->attach(*scale_gamma_b, col+1, row, 1, 1);
+			//entry_gamma_b->set_hexpand(true);
+			entry_gamma_b->set_halign(Gtk::ALIGN_END);
+			controlsGrid->attach(*entry_gamma_b, col+2, row, 1, 1);
+		}
+	}
+	
 	paddedPanel->show_all();
 	return paddedPanel;
 }
